local addonName, AL = ...
local LibParse = LibStub("LibParse")
local LibDeflate = LibStub("LibDeflate")
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)

function AL:ALBD_InitOptionsMenu()
    AL.settings = AL.settings or {}
    AL.settings.Minimap = AL.settings.Minimap or {}
    AL.settings.Minimap.hide = AL.settings.Minimap.hide or false

    local optionsFrameLeaderboard = CreateFrame("Frame", "ALDBExporterOptionsFrame", UIParent)
    optionsFrameLeaderboard.name = "Wow Achievement Leaderboard Options"
    optionsFrameLeaderboard:Hide()

    local category, layout = Settings.RegisterCanvasLayoutCategory(optionsFrameLeaderboard, "Wow Achievement Leaderboard Options")
    category.ID = "AlbdAddonSettings"
    Settings.RegisterAddOnCategory(category)

    optionsFrameLeaderboard:SetScript("OnShow", function (optionsFrameLeaderboard)
        local options = {}
        options.SettingsText = optionsFrameLeaderboard:CreateFontString("SettingsText", "ARTWORK", "GameFontNormal")
        options.SettingsText:SetPoint("TOPLEFT", 0, -16)
        options.SettingsText:SetPoint("LEFT", raidCheckbox, "RIGHT", 0, 1)
        options.SettingsText:SetText("Announcement Settings:")
    
        options.tooltipCheckbox = CreateFrame("CheckButton", "tooltipCheckbox", optionsFrameLeaderboard, "InterfaceOptionsCheckButtonTemplate")
        options.tooltipCheckbox:SetPoint("TOPLEFT", 16, -40)
        --options.tooltipCheckbox:SetChecked(AL.settings.tooltip)
        options.tooltipCheckbox:SetChecked(false)
        options.tooltipCheckbox:SetScript("OnClick", function(self)
            local isChecked = self:GetChecked()
            if isChecked and AL.settings.tooltip == false then
                AL.settings.tooltip = false
            elseif not isChecked and AL.settings.tooltip then
                AL.settings.tooltip = false
            end
        end)
        options.tooltipCheckbox.Text = tooltipCheckbox:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        options.tooltipCheckbox.Text:SetPoint("LEFT", tooltipCheckbox, "RIGHT", 0, 1)
        options.tooltipCheckbox.Text:SetText("Enable tooltip (disabled this release)")
    
        options.guildCheckbox = CreateFrame("CheckButton", "guildCheckbox", optionsFrameLeaderboard, "InterfaceOptionsCheckButtonTemplate")
        options.guildCheckbox:SetPoint("TOPLEFT", 16, -72)
        options.guildCheckbox:SetChecked(AL.settings.guildChat)
        options.guildCheckbox:SetScript("OnClick", function(self)
            local isChecked = self:GetChecked()
            if isChecked and AL.settings.guildChat == false then
                AL.settings.guildChat = true
            elseif not isChecked and AL.settings.guildChat then
                AL.settings.guildChat = false
            end
        end)
        
        options.guildCheckbox.Text = guildCheckbox:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        options.guildCheckbox.Text:SetPoint("LEFT", options.guildCheckbox, "RIGHT", 0, 1)
        options.guildCheckbox.Text:SetText("Enable Guild announcement")
    
        options.raidCheckbox = CreateFrame("CheckButton", "raidCheckbox", optionsFrameLeaderboard, "InterfaceOptionsCheckButtonTemplate")
        options.raidCheckbox:SetPoint("TOPLEFT", 16, -104)
        options.raidCheckbox:SetChecked(AL.settings.raidChat)
        options.raidCheckbox:SetScript("OnClick", function(self)
            local isChecked = self:GetChecked()
            if isChecked and AL.settings.raidChat == false then
                AL.settings.raidChat = true
            elseif not isChecked and AL.settings.raidChat then
                AL.settings.raidChat = false
            end
        end)
        options.raidCheckbox.Text = raidCheckbox:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        options.raidCheckbox.Text:SetPoint("LEFT", options.raidCheckbox, "RIGHT", 0, 1)
        options.raidCheckbox.Text:SetText("Enable Raid announcement")
    
        options.partyCheckbox = CreateFrame("CheckButton", "partyCheckbox", optionsFrameLeaderboard, "InterfaceOptionsCheckButtonTemplate")
        options.partyCheckbox:SetPoint("TOPLEFT", 16, -136)
        options.partyCheckbox:SetChecked(AL.settings.partyChat)
        options.partyCheckbox:SetScript("OnClick", function(self)
            local isChecked = self:GetChecked()
            if isChecked and AL.settings.partyChat == false then
                AL.settings.partyChat = true
            elseif not isChecked and AL.settings.partyChat then
                AL.settings.partyChat = false
            end
        end)
        options.partyCheckbox.Text = partyCheckbox:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        options.partyCheckbox.Text:SetPoint("LEFT", options.partyCheckbox, "RIGHT", 0, 1)
        options.partyCheckbox.Text:SetText("Enable Party announcement")


        options.minimapicon = CreateFrame("CheckButton", "minimapicon", optionsFrameLeaderboard, "InterfaceOptionsCheckButtonTemplate")
        options.minimapicon:SetPoint("TOPLEFT", 16, -168)
        options.minimapicon:SetChecked(AL.settings.Minimap.hide)
        options.minimapicon:SetScript("OnClick", function(self)
            local isChecked = self:GetChecked()
            if isChecked and AL.settings.Minimap.hide == false then
                AL.settings.Minimap.hide = true
                LDBIcon:Hide("ALDB")
            elseif not isChecked and AL.settings.Minimap.hide then
                AL.settings.Minimap.hide = false
                LDBIcon:Show("ALDB")
            end
        end)
        options.minimapicon.Text = minimapicon:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        options.minimapicon.Text:SetPoint("LEFT", options.minimapicon, "RIGHT", 0, 1)
        options.minimapicon.Text:SetText("Hide Minimap icon")

        local function getConfig()
            if AL.settings == nil then AL.settings = ALBDDATA end            
            options.tooltipCheckbox:SetChecked(AL.settings.tooltip)
            options.guildCheckbox:SetChecked(AL.settings.guildChat)
            options.raidCheckbox:SetChecked(AL.settings.raidChat)
            options.partyCheckbox:SetChecked(AL.settings.partyChat)
            options.minimapicon:SetChecked(AL.settings.Minimap.hide)
        end

        optionsFrameLeaderboard.Refresh = function()
            getConfig()
        end

        getConfig()
        
        optionsFrameLeaderboard:SetScript("OnShow", function()
            getConfig()
        end)
        optionsFrameLeaderboard:SetScript("OnHide", function()
            optionsFrameLeaderboard:UnregisterEvent("MODIFIER_STATE_CHANGED")
        end)

    end) 
end

function AL:getAddonSettings()
    return AL.settings
end

function AL:ALDB_initMinimapIcon()
    if LDB then
        local MinimapBtn = LDB:NewDataObject("ALDB", {
            type = "launcher",
			text = "AchievementLeaderboard",
            icon = "Interface\\addons\\AlbdExporter\\icon\\icon",
            OnClick = function(self, button)
				if button == "LeftButton" then
                    if not AL.LeaderboardScreenOpen then
					    AL:showLeaderboard(false)
                    else
                        AL:hideLeaderboard()
                    end
				elseif button == "RightButton" then
					InterfaceOptionsFrame_Show();
					InterfaceOptionsFrame_OpenToCategory("Wow Achievement Leaderboard Options")
				end
			end,
			OnEnter = function(self)
				local text = "|c00FFC100Achievement Leaderboard\nVersion: " .. AL:getVersion() .. "\n\nLeft Click: Show Leaderboard\nRight Click: Open the exporter settings";
				GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
				GameTooltip:SetText(text);
			end,
			OnLeave = function()
				GameTooltip:Hide();
			end
        });
        if LDBIcon then
            LDBIcon:Register("ALDB", MinimapBtn, AL.settings.Minimap);
        end
    end
end