local AddonName, AL = ...
AL.settings = AL.settings or {}
AL.currentCharacter = AL.currentCharacter or {}
AL.tempStore = AL.tempStore or {}
local bnet = C_BattleNet.GetAccountInfoByID(select(3, BNGetInfo()))

-- Initial setup
AL.tempStore = {
    TimePlayed = 0,
    TimePlayedThisLevel = 0,
    Krowi = false,
    Chatprefix = "ALBDexporter"
}

-- Setup character cache
local _, class = UnitClass("player")
local name, realm = UnitName("player")
local realm = realm or GetRealmName()

AL.characterCache = {
    name = name,
    class = class,
    realm = realm,
    gender = UnitSex("player"),
    GUID = UnitGUID("player"),
    level = UnitLevel("player"),
    race = UnitRace("player"),
    faction = UnitFactionGroup("player"),
    btag = bnet["battleTag"]
}

-- Gear slots
local gearSlots = {
    "HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "WristSlot",
    "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot",
    "Trinket0Slot", "Trinket1Slot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot", "AmmoSlot"
}

-- Class colors
AL.ClassColours = {
    UNKNOWN = { r = 1, g = 1, b = 1, colorStr = "#FFFFFF" },
    DRUID = { r = 1, g = 0.49, b = 0.04, colorStr = "#FF7D0A" },
    HUNTER = { r = 0.67, g = 0.83, b = 0.45, colorStr = "#A9D271"},
    MAGE = { r = 0.25, g = 0.78, b = 0.92, colorStr = "#40C7EB"},
    PALADIN = { r = 0.96, g = 0.55, b = 0.73, colorStr = "#F58CBA"},
    PRIEST = { r = 1, g = 1, b = 1, colorStr = "#FFFFFF"},
    ROGUE = { r = 1, g = 0.96, b = 0.41, colorStr = "#FFF569"},
    SHAMAN = { r = 0, g = 0.44, b = 0.87, colorStr = "#0070DE"},
    WARLOCK = { r = 0.53, g = 0.53, b = 0.93, colorStr = "#8787ED"},
    WARRIOR = { r = 0.78, g = 0.61, b = 0.43, colorStr = "#C79C6E"},
    DEATHKNIGHT = { r = 0.77, g = 0.12, b = 0.23, colorStr = "#C41E3A"},
    ["DEATH KNIGHT"] = { r = 0.77, g = 0.12, b = 0.23, colorStr = "#C41E3A"}
}

-- Professions
AL.primaryProfessions = {
    [GetSpellInfo(2259)] = 2259,  -- Alchemy
    [GetSpellInfo(2018)] = 2018,  -- Blacksmithing
    [GetSpellInfo(7411)] = 7411,  -- Enchanting
    [GetSpellInfo(61422)] = 61422, -- Smelting / mining
    [GetSpellInfo(3564)] = 3564,  -- Smelting / mining
    [GetSpellInfo(4036)] = 4036,  -- Engineering
    [GetSpellInfo(2108)] = 2108,  -- Leatherworking
    [GetSpellInfo(3908)] = 3908,  -- Tailoring
    [GetSpellInfo(25229)] = 25229, -- Jewelcrafting
    [GetSpellInfo(45357)] = 45357  -- Inscription
}

AL.secondaryProfessions = {
    [GetSpellInfo(2550)] = 2550,  -- Cooking
    [GetSpellInfo(746)] = 746,    -- First Aid
    [GetSpellInfo(53428)] = 53428, -- Runeforging
    [GetSpellInfo(78670)] = 78670  -- Archeology
}

-- Functions
function AL:getGearslotsAvailable()
    return gearSlots
end

local function setupCharacterDB()
    local character = {
        characterInfo = {
            name = AL.characterCache.name,
            realm = AL.characterCache.realm,
            faction = AL.characterCache.faction,
            guid = AL.characterCache.GUID,
            points = GetTotalAchievementPoints(),
            region = "",
            level = AL.characterCache.level,
            class = AL.characterCache.class,
            race = AL.characterCache.race,
            gender = AL.characterCache.gender,
            played = 0,
            playedLevel = 0,
            battleTag = AL.characterCache.btag
        },
        achievements = {},
        criteria = {},
        categories = {},
        categoryOverview = {},
        statisticData = {},
        statisticCats = {},
        mounts = {},
        pets = {},
        toys = {},
        rep = {},
        professions = {
            primary = {},
            secondary = {}
        },
        armory = {
            talents = "",
            glyphs = {
                major = {},
                minor = {}
            },
            skills = {}
        },
        extra = {
            version = "",
            myCharacters = {}
        },
        guild = {}
    }
    return character
end

local function OnEvent(self, event, arg1)
    if event == "ADDON_LOADED" and arg1 == AddonName then
        if not ALBDDATA then
            ALBDDATA = {
                partyChat = false,
                raidChat = false,
                tooltip = false,
                guildChat = false,
                mainCharacter = ""
            }
        end
        AL.settings = ALBDDATA

        characterData = ALBDCharactersData;
        if not characterData then
            characterData = {};
            ALBDCharactersData = characterData;
        end
        
        AL.currentCharacter = characterData[AL.characterCache.GUID]
        if not AL.currentCharacter then
            AL.currentCharacter = setupCharacterDB()
            characterData[AL.characterCache.GUID] = AL.currentCharacter
        end

        AL:cleanOldCharacters()
        if AL.settings.myCharacters ~= nil then
            local mainCharacter = AL:getMainCharacter()
            AL.settings.mainCharacter = mainCharacter.guid
            AL.settings.myCharacters = nil
        end
        AL:getMyCharacters()
    elseif event == "PLAYER_LOGOUT" then
        ALBDDATA = AL.settings
        ALBDCharactersData[AL.characterCache.GUID] = AL.currentCharacter
    end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_LOGOUT")
frame:SetScript("OnEvent", OnEvent)
