local addonName, AL = ...
local bnet = C_BattleNet.GetAccountInfoByID(select(3, BNGetInfo()))
local LibParse = LibStub("LibParse")
local LibBase64 = LibStub("LibBase64-1.0")
local LibDeflate = LibStub("LibDeflate")
local journalTab = 0

AL.ExportTabOpen = false
AL.LeaderboardScreenOpen = false
AL.Krowi = false
AL.ElvUI = false
AL.LeaderboardData = {}
AL.LastDataUpdate = 0
AL.tempStore.addonIsLoadedIn = false

local cAfter = C_Timer.After
local totalAchieveCompleted = 0
local tradeSkillUIIsLoaded = false
local currentTarget = nil

local mountFaction = {
	["Horde"] = 0,
	["Alliance"] = 1
}

local E, L, V, P, G

local function getGlyphs()
	local minor = {}
	local major = {}
	cAfter(1, function()
	    for t = 1, 6 do
			local active, glyphType, glyphSpellID = GetGlyphSocketInfo(t)
			if active and glyphSpellID then
				local localizedName = GetSpellInfo(glyphSpellID)
				if localizedName then
					if glyphType == 1 then
						table.insert(major, {name = localizedName, spellId = glyphSpellID})
					elseif glyphType == 2 then
						table.insert(minor, {name = localizedName, spellId = glyphSpellID})
					end
				end
			end
			if AL.currentCharacter.armory.glyphs.major == nil or AL.currentCharacter.armory.glyphs.minor == nil then
				AL.currentCharacter.armory.glyphs = {
					major = {},
					minor = {}
				}
			end
			AL.currentCharacter.armory.glyphs.major = major
			AL.currentCharacter.armory.glyphs.minor = minor
			AL.LastDataUpdate = time()
		end
	end)
end


function AL:BuildNewTab(name, text, helptip, loadFunc, filter)
	local numtabs, tab = 0
	repeat
	  numtabs = numtabs + 1
	until (not _G["AchievementFrameTab"..numtabs])
	tab = CreateFrame("Button", "AchievementFrameTab"..numtabs, AchievementFrame, "AchievementFrameTabButtonTemplate")

	if AL.Krowi ~= true then
		tab:SetPoint("LEFT", "AchievementFrameTab"..numtabs-1, "RIGHT", -5, 0)
		tab:SetID(numtabs)
		tab:SetScript("OnClick", function(self, button, down)
			AL:showLeaderboard(true)
		end)
	else
		cAfter(0.5, function()
			repeat
			  numtabs = numtabs + 1
			until (not _G["AchievementFrameTab"..numtabs])
			tab:SetID(numtabs)
			tab:SetPoint("LEFT", "AchievementFrameTab"..numtabs-1, "right", 65, 0)
			tab:SetFrameLevel(tab:GetFrameLevel() + 2);
			tab:SetScript("OnClick", function(self, button, down)
				PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB);
				AL:showLeaderboard(true)
			end)
			tab.selectedAchievement = nil
		end)
	end
	tab:SetText(text)

	if AL.ElvUI == true then
		tab:StripTextures()
		tab:SetTemplate("Transparent")
		tab:SetBackdropColor(unpack(E.media.backdropcolor))
		tab:SetBackdropBorderColor(unpack(E.media.bordercolor))
		tab:GetFontString():SetJustifyV("BOTTOM")
	end
	PanelTemplates_SetNumTabs(AchievementFrame, numtabs)
end

function getFactionData()
	local factionData = {};
	local factionIndex = 1
	local lastFactionName
	repeat
		local name, description, standingId, bottomValue, topValue, earnedValue, atWarWith,	canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID = GetFactionInfo(factionIndex)
		if name == lastFactionName then break end
		lastFactionName  = name

		if name ~= nil then
			tinsert(factionData, {id = factionID, standingId = standingId, bottomValue = bottomValue, topValue = topValue, earnedValue = earnedValue, isHeader = isHeader, isChild = isChild});			
		end
		factionIndex = factionIndex + 1
	until factionIndex > 200
	return factionData
end

local function statistics_MakeCategoryList(source)
	local categories = {};
	for i, id in next, source do
		local _, parent = GetCategoryInfo(id);
		if ( parent == -1 or parent == GUILD_CATEGORY_ID ) then
			tinsert(categories, { id = id });
		end
	end
	local _, parent;
	for i = #source, 1, -1 do
		_, parent = GetCategoryInfo(source[i]);
		for j, category in next, categories do
			if ( category.id == parent ) then
				category.parent = 0;
				category.collapsed = true;
				local elementData = {
					id = source[i],
					parent = category.id,
					hidden = true,
					isChild = (type(category.id) == "number"),
				};
				tinsert(categories, j+1, elementData);
			end
		end
	end
	return categories;
end

local function getStatisticValue(value)
	if type(value) == "number" then
		return value
	end
	
	local containsTextureTags = value:find("|T.-|t")

	if containsTextureTags then
		local plainValue = value:gsub("|T.-|t", "")
		plainValue = plainValue:gsub("%s+", "")
		while #plainValue < 4 do
			plainValue = "0" .. plainValue
		end
		return plainValue
	else
		return value
	end
end

local function getStatistic(type)
	local catlist = statistics_MakeCategoryList(GetStatisticsCategoryList())
	local statisticsData = {}
	local statisticsCat = {}
	for i = 1, #catlist do
		local catid = 0;
		local catName = "";
		local parentid = 0;
		for k, v in pairs(catlist[i]) do
			if k == "id" then
				catid = v
				catName = GetCategoryInfo(v)
				for i=1, GetCategoryNumAchievements(v) do
					local quantity, skip, id = GetStatistic(v, i);
					if ( not skip ) then
						_, statname = GetAchievementInfo(id)
						if quantity == "--" then quantity = -1 end
						table.insert(statisticsData, {id = id, amount = getStatisticValue(quantity), categoryid = v})
					end
				end
			end
			if k == "parent" then
				parentid = v
			end
		end
		table.insert(statisticsCat, {id = catid, parent = parentid})
	end
	if type == "cat" then
		return statisticsCat
	else
		return statisticsData
	end
end

function AL:updateAchievements()
	AL.currentCharacter.achievements = AL:getAchievements()
	AL.currentCharacter.criteria = AL:getMoreCritInfo()
	AL.currentCharacter.categories = AL:getAchievementCategories()
	AL.currentCharacter.categoryOverview = AL:getAchievementOverview()
	AL.currentCharacter.characterInfo.points = GetTotalAchievementPoints()
end

local function FixBeforeExport()
	AL.currentCharacter.pets = AL:getPets()
	AL.currentCharacter.toys =  AL:getToys()
	AL.currentCharacter.mounts =  AL:getMounts()
	AL.currentCharacter.extra.myCharacters = AL.tempStore.myCharacters
	local totalPoints = GetTotalAchievementPoints()
	if totalPoints > 0 and AL.currentCharacter.characterInfo.points ~= totalPoints then
		AL.currentCharacter.characterInfo.points = GetTotalAchievementPoints()
	end
	AL.currentCharacter.characterInfo.level = UnitLevel("player")
	AL.currentCharacter.achievements = AL:getAchievements()
	AL.currentCharacter.criteria = AL:getMoreCritInfo()
	AL.currentCharacter.statisticData = getStatistic("data")
	AL.currentCharacter.categories = AL:getAchievementCategories()
	AL.currentCharacter.categoryOverview = AL:getAchievementOverview()
	AL.currentCharacter.rep = getFactionData()
	if AL.currentCharacter.armory.skills == "" then
		AL.currentCharacter.armory.skills = {}
	end
end

function AL:getExportData()
	FixBeforeExport()
	local jsondata = LibParse:JSONEncode(AL.currentCharacter)
	local compressed = LibDeflate:CompressDeflate(LibParse:JSONEncode(AL.currentCharacter))
	local encodedforprint = LibDeflate:EncodeForPrint(compressed)
	AL.currentCharacter.lastExport = time()
	return encodedforprint
end

local EventFrame = CreateFrame("Frame");
EventFrame:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
EventFrame:RegisterEvent("UNIT_TARGET");
EventFrame:RegisterEvent("INSPECT_ACHIEVEMENT_READY");
EventFrame:RegisterEvent("PLAYER_TALENT_UPDATE");
EventFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
EventFrame:RegisterEvent("EQUIPMENT_SWAP_FINISHED");
EventFrame:RegisterEvent("REPLACE_ENCHANT");
EventFrame:RegisterEvent("TRADE_SKILL_SHOW");
EventFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
EventFrame:RegisterEvent('ADDON_LOADED')
EventFrame:RegisterEvent('PLAYER_LOGIN')
EventFrame:RegisterEvent('PLAYER_LOGOUT')
EventFrame:RegisterEvent('PLAYER_ENTERING_WORLD')
EventFrame:RegisterEvent('TIME_PLAYED_MSG')
EventFrame:RegisterEvent('ACHIEVEMENT_EARNED')
EventFrame:RegisterEvent('SPELLS_CHANGED')
EventFrame:RegisterEvent('TRADE_SKILL_CLOSE')
EventFrame:RegisterEvent("PLAYER_GUILD_UPDATE")
EventFrame:RegisterEvent("GUILD_ROSTER_UPDATE")
EventFrame:RegisterEvent('NEW_PET_ADDED')
EventFrame:RegisterEvent('NEW_TOY_ADDED')
EventFrame:RegisterEvent('NEW_MOUNT_ADDED')
EventFrame:RegisterEvent('CVAR_UPDATE')



EventFrame:SetScript('OnEvent', function(self, event, ...)
    if self[event] then
        self[event](self, ...)
    end
end)

function EventFrame:PLAYER_LOGIN(arg1, arg2, ...)
	if IsAddOnLoaded("Krowi_AchievementFilter") then
		AL.Krowi = true;
	end
	if IsAddOnLoaded("ElvUI") then
		E, L, V, P, G = unpack(ElvUI)
		AL.ElvUI = true;
	end
end

function EventFrame:CVAR_UPDATE(arg1, arg2, ...)
	if arg1 == 'petJournalTab' and arg2 ~= journalTab and (AL.tempStore.companionupdate == nill or AL.tempStore.companionupdate < (time() - 90000)) then
		AL.currentCharacter.pets = AL:getPets()
		AL.currentCharacter.toys =  AL:getToys()
		AL.currentCharacter.mounts =  AL:getMounts()
		journalTab = arg2
		AL.tempStore.companionupdate = time()
	end
end

function EventFrame:GUILD_ROSTER_UPDATE(arg1, arg2, ...)
	if (IsInGuild()) then
		AL:getGuildInfo()
	end
end

function EventFrame:PLAYER_GUILD_UPDATE(arg1, arg2, ...)
	if (IsInGuild()) then
		AL:getGuildInfo()
	end
end

function EventFrame:NEW_PET_ADDED(...)
	if #AL.currentCharacter.pets == 0 or #AL.currentCharacter.pets < #AL:getPets() then
		AL.currentCharacter.pets = AL:getPets()
	end
end

function EventFrame:NEW_TOY_ADDED(...)
	if #AL.currentCharacter.toys == 0 or #AL.currentCharacter.toys < # AL:getToys() then
		AL.currentCharacter.toys =  AL:getToys()
	end
end

function EventFrame:NEW_MOUNT_ADDED(...)
	if #AL.currentCharacter.mounts == 0 or #AL.currentCharacter.mounts < #AL:getMounts() then
		AL.currentCharacter.mounts =  AL:getMounts()
	end
end

function EventFrame:PLAYER_ENTERING_WORLD(arg1, arg2, ...)
	EventFrame:UnregisterEvent('PLAYER_ENTERING_WORLD')

	AL:validateGuild()

	if #AL.currentCharacter.achievements == 0 then
		AL.currentCharacter.achievements = AL:getAchievements()
	end

	if #AL.currentCharacter.categories == 0 then
		AL.currentCharacter.categories = AL:getAchievementCategories()
	end

	if #AL.currentCharacter.categoryOverview == 0 then
		AL.currentCharacter.categoryOverview = AL:getAchievementOverview()
	end

	if #AL.currentCharacter.criteria == 0 then
		AL.currentCharacter.criteria = AL:getMoreCritInfo()
	end

	if #AL.currentCharacter.statisticData == 0 then
		AL.currentCharacter.statisticData = getStatistic("data")
		AL.currentCharacter.statisticCats = getStatistic("cat")
	end

	if #AL.currentCharacter.mounts < #AL:getMounts() then
		AL.currentCharacter.mounts = AL:getMounts()
	end
	
	if #AL.currentCharacter.pets < #AL:getPets() then
		AL.currentCharacter.pets = AL:getPets()
	end
	
	if #AL.currentCharacter.rep == 0 then
		AL.currentCharacter.rep = getFactionData()
	end
	
	if #AL.currentCharacter.toys < #AL:getToys() then
		AL.currentCharacter.toys = AL:getToys()
	end

	if AL.currentCharacter.characterInfo.points ~= GetTotalAchievementPoints() then
		AL.currentCharacter.characterInfo.points = GetTotalAchievementPoints()
	end

	getGlyphs()
	
	AL.tempStore.addonIsLoadedIn = true
end

function EventFrame:ADDON_LOADED(arg1, arg2, ...)
	if arg1 == "Blizzard_AchievementUI" and AL.Krowi == true then
		frame, panel = AL:BuildNewTab("ALBD_ExportFrame", "Export", "Export Achievement", OnLoad)
	elseif arg1 == "Blizzard_AchievementUI" and AL.Krowi ~= true then
		frame, panel = AL:BuildNewTab("ALBD_ExportFrame", "Export", "Export Achievement", OnLoad)
	elseif arg1 == addonName then
		AL:ALBD_InitOptionsMenu()
		AL:ALDB_initMinimapIcon()
		cAfter(3, function()
			AL:sendLeaderboardMessage(AL:getVersion() .. " is Loaded")
			AL:RequestTimePlayed()
			if #AL.currentCharacter.professions.primary == 0 then
				AL:sendLeaderboardMessage("Open your profession windows too load in your recipes (twice)")
			end
			AL.currentCharacter.extra.version = "v1.4.7"
		end)
	end
end
function EventFrame:TIME_PLAYED_MSG(TimePlayed, TimePlayedThisLevel, ...)
	if AL.currentCharacter.characterInfo then
		if TimePlayed == nil or TimePlayedThisLevel == nil then return end
		if AL.currentCharacter.characterInfo.played == 0 or AL.currentCharacter.characterInfo.playedLevel == "" or tonumber(TimePlayed) > tonumber(AL.currentCharacter.characterInfo.played) then
			AL.currentCharacter.characterInfo.played = TimePlayed
		end
		if AL.currentCharacter.characterInfo.playedLevel == 0 or AL.currentCharacter.characterInfo.playedLevel == "" or tonumber(TimePlayedThisLevel) > tonumber(AL.currentCharacter.characterInfo.playedLevel) then
			AL.currentCharacter.characterInfo.playedLevel = TimePlayedThisLevel
		end
	end
end
function EventFrame:ACHIEVEMENT_EARNED(arg1, ...)
	local IDNumber, Name, Points, Completed, Month, Day, Year, _, _, _, _, isGuild, earnedByMe, earnedBy, isStatistic = GetAchievementInfo(arg1)
	if Points ~= nil and not isGuild then
		AL:sendAchievementAnnouncement("Achievement completed " .. Name .. " +" .. Points .. " points (total " .. GetTotalAchievementPoints()  .. " points)")
		AL:updateAchievement(arg1)
	end
end


local function getTradeProfession()
    local typeSkill = 0;
    if IsTradeSkillLinked and IsTradeSkillLinked() then
        return;
    end
    if C_TradeSkillUI then
        if (C_TradeSkillUI.IsTradeSkillLinked and C_TradeSkillUI.IsTradeSkillLinked())
        or (C_TradeSkillUI.IsTradeSkillGuild and C_TradeSkillUI.IsTradeSkillGuild()) then
            return;
        end
    end
    tradeSkillUIIsLoaded = true
    local tradeskillName, skill, maxskill = GetTradeSkillLine()
    if not tradeskillName or tradeskillName == "UNKNOWN" then return end
    for index, data in pairs(AL.primaryProfessions) do
        if index == tradeskillName then
            tradeskillId = data
            typeSkill = 1;
        end
    end
    for index, data in pairs(AL.secondaryProfessions) do
        if index == tradeskillName then
            tradeskillId = data
            typeSkill = 2;
        end
    end
    local numTradeSkills = GetNumTradeSkills()
    if not numTradeSkills or numTradeSkills == 0 then return end
    local skillName, skillType = GetTradeSkillInfo(1)
    if skillType ~= "header" then return end

    local link
    local header = "";
    local recipes = {}
    for i = 1, numTradeSkills do
        skillName, skillType = GetTradeSkillInfo(i)
        if skillType == "header" or skillType == "subheader" then
            header = skillName
        else
            link = GetTradeSkillRecipeLink(i)
            craftInfo = tonumber(link:match("enchant:(%d+)"))
            table.insert(recipes, {spellId = craftInfo, fullLink = craftInfo, header = header})
        end
    end

    local tradeskill = {
        id = tradeskillId,
        typeSkill = typeSkill,
        skilllvl = skill,
        maxlvl = maxskill,
        recipes = recipes
    }

    local tradeSkillExist = false
    local tradeSkillIndex = nil
    if #recipes > 0 then
        if typeSkill == 1 then
            local primary = AL.currentCharacter.professions.primary
            if #primary > 2 then
                AL.currentCharacter.professions.primary = {}
            end
            for index, data in pairs(primary) do
                if data.id == tradeskillId then
                    tradeSkillExist = true
                    tradeSkillIndex = index
                end
            end
            if #primary <= 2 and not tradeSkillExist then
                table.insert(primary, tradeskill)
                AL.LastDataUpdate = time()
                AL:sendLeaderboardMessage("Added recipes for " .. tradeskillName)
            elseif #primary <= 2 and tradeSkillExist then
                if not primary[tradeSkillIndex].skilllvl or primary[tradeSkillIndex].skilllvl < skill then
                    primary[tradeSkillIndex].skilllvl = skill
                end
                if not primary[tradeSkillIndex].maxlvl or primary[tradeSkillIndex].maxlvl < maxskill then
                    primary[tradeSkillIndex].maxlvl = maxskill
                end
                if #primary[tradeSkillIndex].recipes < #recipes then
                    primary[tradeSkillIndex].recipes = recipes
                    AL.LastDataUpdate = time()
                    AL:sendLeaderboardMessage("Added recipes for " .. tradeskillName)
                end
            end
        elseif typeSkill == 2 then
            local secondary = AL.currentCharacter.professions.secondary
            if #secondary > 3 then
                AL.currentCharacter.professions.secondary = {}
            end
            for index, data in pairs(secondary) do
                if data.id == tradeskillId then
                    tradeSkillExist = true
                    tradeSkillIndex = index
                end
            end
            if #secondary <= 3 and not tradeSkillExist then
                table.insert(secondary, tradeskill)
                AL.LastDataUpdate = time()
                AL:sendLeaderboardMessage("Added recipes for " .. tradeskillName)
            elseif #secondary <= 3 and tradeSkillExist then
                if not secondary[tradeSkillIndex].skilllvl or secondary[tradeSkillIndex].skilllvl < skill then
                    secondary[tradeSkillIndex].skilllvl = skill
                end
                if not secondary[tradeSkillIndex].maxlvl or secondary[tradeSkillIndex].maxlvl < maxskill then
                    secondary[tradeSkillIndex].maxlvl = maxskill
                end
                if #secondary[tradeSkillIndex].recipes < #recipes then
                    secondary[tradeSkillIndex].recipes = recipes
                    AL.LastDataUpdate = time()
                    AL:sendLeaderboardMessage("Added recipes for " .. tradeskillName)
                end
            end
        end
    end
    tradeSkillUIIsLoaded = false
end

local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("TRADE_SKILL_SHOW")
EventFrame:RegisterEvent("TRADE_SKILL_CLOSE")

-- Assign event handlers
EventFrame:SetScript("OnEvent", function(self, event, ...)
    if event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" then
        getTradeProfession()
    end
end)

function AL:listAllKnownPlayers()
	AL.knownPlayersCache = {}
	for guidId, data in pairs(ALBDCharactersData) do
		if ALBDCharactersData[guidId].characterInfo ~= nil and ALBDCharactersData[guidId].characterInfo.points ~= nil and ALBDCharactersData[guidId].characterInfo.points > 0 then
			if ALBDCharactersData[guidId].characterInfo.name ~= nil then
				if ALBDCharactersData[guidId].characterInfo.realm == AL.currentCharacter.characterInfo.realm then
					local class = ""
					if ALBDCharactersData[guidId].characterInfo.class ~= nil and ALBDCharactersData[guidId].characterInfo ~= "" then
						class = ALBDCharactersData[guidId].characterInfo.class
					else
						class = "UNKNOWN"
					end
					table.insert(AL.knownPlayersCache, {name = ALBDCharactersData[guidId].characterInfo.name, points = ALBDCharactersData[guidId].characterInfo.points, updated = date("%d-%m-%Y %H:%M:%S", ALBDCharactersData[guidId].characterInfo.time), class = class, realm = ALBDCharactersData[guidId].characterInfo.realm})
				end
			end
		end
	end
	table.sort(AL.knownPlayersCache, function(a,b) return a.points > b.points end)
end


function AL:getActualPoints(guid)
	if guid ~= nil then
		character = ALBDCharactersData[guid];
		if character ~= nil then
			local actualPoints = 0
			for index, data in pairs(character.achievements) do
				if data.completed and data.earnedByMe then
					local _, _, points = GetAchievementInfo(data.id)
					if points ~= nil then
						actualPoints = actualPoints + points
					end
				end
			end
			character.characterInfo.actualPoints = actualPoints
			return actualPoints
		end
	else
		local actualPoints = 0
		if AL.currentCharacter.achievements ~= nil then
			for index, data in pairs(AL.currentCharacter.achievements) do
				local id, name, points, completed, _, _, _, _, _, _, _, _, earnedByMe, earnedBy = GetAchievementInfo(data.id)
				if points ~= nil and completed and earnedByMe then
					actualPoints = actualPoints + points
				end
			end
			AL.currentCharacter.characterInfo.actualPoints = actualPoints
		end
	end
end

local function getGuildInfo()
	local guildPoints = GetTotalAchievementPoints(true)
	GetGuildAchievements()
	total, completed = GetNumCompletedAchievements(true)
	print("Total Guild achievments:", total, "completedGuildAchievements:", completed)
end

local function albdCommands(msg, editbox)
	if msg == 'export' then
		AL:showLeaderboard(true)
	elseif msg == 'settings' then
		InterfaceOptionsFrame_OpenToCategory("Wow Achievement Leaderboard Options")
	elseif msg == 'show' then
		AL:showLeaderboard(false)
	elseif msg == 'achievements' then
		AL:updateAchievements()
	elseif msg == 'guild' then
		AL:getGuildInfo()
 	else
		DEFAULT_CHAT_FRAME:AddMessage("|c006969FF[AL] :|r Cata Achievement Leaderboard |r");
		DEFAULT_CHAT_FRAME:AddMessage("|c006969FF[AL] :|r use /leaderboard export |r");
		DEFAULT_CHAT_FRAME:AddMessage("|c006969FF[AL] :|r use /leaderboard settings |r");
		DEFAULT_CHAT_FRAME:AddMessage("|c006969FF[AL] :|r use /leaderboard achievements (update data if invalid) ");
	end
end

SLASH_ALBD1, SLASH_ALBD2, SLASH_ALBD3 = '/leaderboard', '/LEADERBOARD', '/albd'
SlashCmdList["ALBD"] = albdCommands