local addonName, AL = ...
local guildUpdated = false

function AL:getCurrentGuild()
	if AL.tempStore.addonIsLoadedIn then
		if IsInGuild() and guildUpdated == false then
			local guildName, guildRankName, guildRankIndex = GetGuildInfo(AL.characterCache.name);
			if guildName ~= nil and (AL.currentCharacter.characterInfo.guild ~= guildName or AL.currentCharacter.characterInfo.guild == "") then
				AL.currentCharacter.characterInfo.guild = guildName
				AL.currentCharacter.characterInfo.guildRank = guildRankName
				AL.currentCharacter.characterInfo.guildRankId = guildRankIndex
				guildUpdated = true
			end
		else
			AL:validateGuild()
			guildUpdated = true
		end
	end
end

function AL:validateGuild()
	if not IsInGuild() then
		if AL.currentCharacter.guild.name ~= nil or AL.currentCharacter.characterInfo.guild ~= "" then
			AL.currentCharacter.guild = {}
			AL.currentCharacter.characterInfo.guild = ""
			AL.currentCharacter.characterInfo.guildRankName = ""
			AL.currentCharacter.characterInfo.guildRankId = ""
		end
	else
		if AL.currentCharacter.guild ~= nil and AL.currentCharacter.characterInfo.guild ~= AL.currentCharacter.guild.name then
			local guildName, guildRankName, guildRankIndex = GetGuildInfo(AL.characterCache.name);
			if guildName ~= nil and (AL.currentCharacter.characterInfo.guild == nil or AL.currentCharacter.characterInfo.guild == "") then
				AL.currentCharacter.characterInfo.guild = guildName
				AL.currentCharacter.characterInfo.guildRank = guildRankName
				AL.currentCharacter.characterInfo.guildRankId = guildRankIndex
			end			
		end
	end
end

-- achievement info:
function AL:getAchievementOverview()
	local categories = GetCategoryList();
	local totalAchievements = 0;
	local achievementOverview = {}
	for _, categoryID in ipairs(categories) do
		local Title, ParentCategoryID = GetCategoryInfo(categoryID);
		if ParentCategoryID == -1 then
			local totalTopCategory = 0;
			local totalCompletedTopCategory = 0;
			total, completed, incompleted = GetCategoryNumAchievements(categoryID, true)
			totalTopCategory = totalTopCategory + total;
			totalCompletedTopCategory = totalCompletedTopCategory + completed;
			for _, subCatID in ipairs(categories) do
				local subCatName, subCatParentID = GetCategoryInfo(subCatID);
				if subCatParentID == categoryID then
					local total2, completed2, incompleted2 = GetCategoryNumAchievements(subCatID, true)
					totalTopCategory = totalTopCategory + total2;
					totalCompletedTopCategory = totalCompletedTopCategory + completed2;
				end
			end
			if categoryID ~= 81 then
				totalAchievements = totalAchievements + totalTopCategory;
			end
			table.insert(achievementOverview, {id = categoryID, subcat = ParentCategoryID, completed = totalCompletedTopCategory, total = totalTopCategory});
		end
	end	
	return achievementOverview;
end

function AL:getAchievements()
	local totalAchieveCompleted = 0
	local categories = GetCategoryList();
	local achievementsList = {};
	for _, categoryID in ipairs(categories) do
		local Title, ParentCategoryID = GetCategoryInfo(categoryID);
		total, completed, incompleted = GetCategoryNumAchievements(categoryID)
		for i=1, (GetCategoryNumAchievements(categoryID)) do
			local id, name, _, completed, month, day, year, _, _, _, _, isGuild, earnedByMe, earnedBy, isStatistic = GetAchievementInfo(categoryID, i)
            local completedate = "";
			if(day and month and year) then
				completedate = day .. "/"  .. month .. "/" .. year
				totalAchieveCompleted = totalAchieveCompleted + 1
			end
			table.insert(achievementsList, {id = id, completed = completed, category = categoryID, linked = 0, completedate = completedate, earnedByMe = earnedByMe, earnedBy = earnedBy})
			previousid = id
			if completed and GetPreviousAchievement(id) then
				while GetPreviousAchievement(previousid) do
					local previousachievement = GetPreviousAchievement(previousid)
					local previd, prevname, prevpoints, prevcompleted, prevmonth, prevday, prevyear, _, _, _, _, isGuild, earnedByMe, earnedBy, isStatistic = GetAchievementInfo(previousachievement)		
                    local completedate = "";
					if(prevday and prevmonth and prevyear) then
						completedate = prevday .. "/"  .. prevmonth .. "/" .. prevyear
						totalAchieveCompleted = totalAchieveCompleted + 1
					end
					table.insert(achievementsList, {id = previd, completed = prevcompleted, category = categoryID, linked = previousid, completedate = completedate, earnedByMe = earnedByMe, earnedBy = earnedBy})
					previousid = previd
				end
			end
		end
	end
	return achievementsList;
end

function AL:updateAchievement(achievementid)
	for index, data in pairs(AL.currentCharacter.achievements) do
		if(data.id == achievementid) then
			local id, name, _, completed, month, day, year, _, _, _, _, isGuild, earnedByMe, earnedBy, isStatistic = GetAchievementInfo(data.id)
			if completed and month and day and year then
				local completedate = day .. "/"  .. month .. "/" .. year
				AL.currentCharacter.achievements[index].completed = completed;
				AL.currentCharacter.achievements[index].completedate = completedate;
				AL.currentCharacter.achievements[index].earnedbyme = earnedByMe
				AL.currentCharacter.achievements[index].earnedby = earnedBy
				if data.category ~= nil and data.category ~= "" then
					AL:updateAchievementCategory(data.category)
				end
			end
			AL.currentCharacter.categoryOverview = AL:getAchievementOverview()
			AL.currentCharacter.criteria = AL:getMoreCritInfo()
		end
	end
end

function AL:updateAchievementCategory(categoryid)
	for index2, category in pairs(AL.currentCharacter.categories) do
		if category.id == categoryid then
			local catTotal, catCompleted, catIncompleted = GetCategoryNumAchievements(categoryid, true)
			AL.currentCharacter.categories[index2].completed = catCompleted
			if category.subcat ~= nil or category.subcat ~= "" then
				AL:updateAchievementCategory(category.subcat)
			end
		end
	end
end

function AL:getMoreCritInfo()
	local achievements = AL.currentCharacter.achievements
	local creteriaList = {}
	for index, data in pairs(achievements) do
		if GetAchievementNumCriteria(index) > 0 then
			local een, twee, drie, vier, vijf, zes, zeven = GetAchievementInfo(index)
			local criteriaValues = {}
			criteriaValues["achievementid"] = index
			criteriaValues["completed"] = vier
			criteriaValues["total"] = GetAchievementNumCriteria(index)
			criteriaValues["criterias"] = {}
			for i=1, (GetAchievementNumCriteria(index)) do
				criteriaString, criteriaType, completed, quantity, reqQuantity, charName, flags, assetID, quantityString, criteriaID = GetAchievementCriteriaInfo(index, i)
				table.insert(criteriaValues["criterias"], {criteriaType = criteriaType, completed = completed, quantity = quantity, reqQuantity = reqQuantity, charName = charName, assetID = assetID, quantityString = quantityString, criteriaID = criteriaID})
			end
			table.insert(creteriaList, criteriaValues)
		end
	end
	return creteriaList
end

function AL:getAchievementCategories()
	local categories = GetCategoryList();
	local achievementCategories = {};
	for _, categoryID in ipairs(categories) do
		local Title, ParentCategoryID = GetCategoryInfo(categoryID);
		if ParentCategoryID == -1 then
			total, completed, incompleted = GetCategoryNumAchievements(categoryID, true)
			for _, subCatID in ipairs(categories) do
				local subCatName, subCatParentID = GetCategoryInfo(subCatID);
				if subCatParentID == categoryID then
					local total2, completed2, incompleted2 = GetCategoryNumAchievements(subCatID, true)
					table.insert(achievementCategories, {id = subCatID, subcat = subCatParentID, completed = completed2, total = total2});
				end
			end
			table.insert(achievementCategories, {id = categoryID, subcat = 0, completed = completed, total = total});
		end
	end	
	return achievementCategories;
end

function AL:getTotalCompletedAchievements()
	local totalCompleted = 0
	local FoS = 0
	local completedByMe = 0
	for index, data in pairs(AL.currentCharacter.achievements) do
		if data.completed and data.category ~= 81 then
			totalCompleted = totalCompleted+1
			if data.earnedByMe then
				completedByMe = completedByMe + 1
			end
		elseif data.completed and data.category == 81 then
			FoS = FoS+1
		end
	end
	return totalCompleted, FoS, completedByMe
end


-- Companion Data:
function AL:getMounts()
	local AllMounts = C_MountJournal.GetMountIDs()
	local mounts = {}
	for index=1,#AllMounts do
		local name, spellID, icon, isActive, isUsable, sourceType, isFavorite, isFactionSpecific, faction, shouldHideOnChar, isCollected, mountID = C_MountJournal.GetMountInfoByID(AllMounts[index])
		if isCollected == true and not shouldHideOnChar then
			table.insert(mounts, {id = mountID, companionType = "MOUNT", spellid = spellID, faction = faction, collected = isCollected})
		end
	end 
	return mounts
end

function AL:getPets()
	local numPets, numOwned = C_PetJournal.GetNumPets()
	local pets = {}
	for index = 1, numOwned do
		local petID, speciesID, owned, customName, level, favorite, isRevoked, speciesName, icon, petType, companionID, tooltip, description, isWild, canBattle, isTradeable, isUnique, obtainable = C_PetJournal.GetPetInfoByIndex(index)
		if owned then
			table.insert(pets, {id = companionID, species = speciesID, companionType = "CRITTER"})
		end
	end
	return pets
end

function AL:getToys()
	local numToys = C_ToyBox.GetNumToys()
	local toys = {}
	for index = 1, numToys do	
		local itemID = C_ToyBox.GetToyFromIndex(index)
		local itemID, toyName, icon, isFavorite, hasFanfare, itemQuality = C_ToyBox.GetToyInfo(itemID)
		if itemID and PlayerHasToy(itemID) then
			table.insert(toys, {id = itemID, companionType = "TOYS"})
		end
	end
	return toys
end


-- Borrowed from rating buster!!
-- As of Classic Patch 3.4.0, GetTalentInfo indices no longer correlate
-- to their positions in the tree. Building a talent cache ordered by
-- tier then column allows us to replicate the previous behavior.
local orderedTalentCache = {}
do
	local f = CreateFrame("Frame")
	f:RegisterEvent("SPELLS_CHANGED")
	f:SetScript("OnEvent", function()
		local temp = {}
		for tab = 1, GetNumTalentTabs() do
			temp[tab] = {}
			local products = {}
			for i = 1,GetNumTalents(tab) do
				local name, _, tier, column = GetTalentInfo(tab,i)
				local product = (tier - 1) * 4 + column
				temp[tab][product] = i
				table.insert(products, product)
			end

			table.sort(products)

			orderedTalentCache[tab] = {}
			local j = 1
			for _, product in ipairs(products) do
				orderedTalentCache[tab][j] = temp[tab][product]
				j = j + 1
			end
		end
		f:UnregisterEvent("SPELLS_CHANGED")
	end)
end

function AL:getTalents()
	cAfter(2, function()
		local talents = {}
		local numTabs = GetNumTalentTabs()
		for t = 1, numTabs do
			local numTalents = GetNumTalents(t)
			for i = 1, numTalents do
				local nameTalent, icon, tier, column, currRank, maxRank = GetOrderedTalentInfo(t, i)
				if currRank ~= nil then
					table.insert(talents, tostring(currRank))
				end
			end
			if (t < 3) then
				table.insert(talents, "-")
			end
		end
		if table.concat(talents) ~= "--" and AL.currentCharacter.armory.talents ~= table.concat(talents) then
			AL.currentCharacter.armory.talents = table.concat(talents)
			AL.LastDataUpdate = time()
			AL:sendLeaderboardMessage("Talents has been updated")
		end
	end)
end

local function GetOrderedTalentInfo(tab, num)
    if tab ~= nil and num ~= nil then
        if orderedTalentCache[tab] then
	        return GetTalentInfo(tab, orderedTalentCache[tab][num])
        end
    end
end

local GUILD_FEAT_OF_STRENGTH_ID = 15093;
local GUILD_CATEGORY_ID = 15076;
local ACHIEVEMENTUI_GUILDSUMMARYCATEGORIES = {15088, 15077, 15078, 15079, 15080, 15089};

local function GetGuildAchievements()
	local guildCategories = {}
	local guildAchievements = {}

    local categories = GetGuildCategoryList()

    for _, categoryID in ipairs(categories) do
        local name, parentID = GetCategoryInfo(categoryID)
		local total, completed, incompleted = GetCategoryNumAchievements(categoryID, true)
		table.insert(guildCategories, {id = categoryID, parent = parentID, total = total, completed = completed, incompleted = incompleted})
		local numAchievements = GetCategoryNumAchievements(categoryID)
		for i = 1, numAchievements do
			local achievementID, name, points, completed, month, day, year, description, flags, icon, rewardText = GetAchievementInfo(categoryID, i)
			if achievementID then
				local completedate = "";
				if(day and month and year) then
					completedate = day .. "/"  .. month .. "/" .. year
				end
				table.insert(guildAchievements, {id = achievementID, completed = completed, date = completedate, category = categoryID, parentId = parentID})
			end
		end
    end

    return guildCategories, guildAchievements
end

function AL:getGuildInfo()
    if (IsInGuild()) then
        local guildName, guildRankName, guildRankIndex = GetGuildInfo(AL.characterCache.name);
		if guildName ~= nil then
            local guildCategories, guildAchievements = GetGuildAchievements()
			local total, completed = GetNumCompletedAchievements(true)
            if AL.currentCharacter.guild == nil then
                AL.currentCharacter.guild = {
                    info = {
						name = "",
						completed = 0,
						total = 0,
						points = 0
					},
                    achievements = {},
                    categories = {}
                }
            end
            AL.currentCharacter.guild.info = {
				name = guildName,
				completed = completed,
				total = total,
				points = GetTotalAchievementPoints(true)
			}
            AL.currentCharacter.guild.achievements = guildAchievements
            AL.currentCharacter.guild.categories = guildCategories
			if DevTool and self.DEBUG then
				DevTool:AddData(guildCategories, "guildCategories")
				DevTool:AddData(guildAchievements, "guildAchievements")
				DevTool:AddData(AL.currentCharacter.guild.info, "guildInfo")
			end
		end
		return true
	else
		return false
    end

end